/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaxonSoundFactSheetAndroidController.java,v c90f4569fa41 2010/07/04 02:23:58 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.taxonomy.factsheet.sound;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.zip.GZIPInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import android.app.Activity;
import android.content.res.AssetManager;
import it.tidalwave.bluebill.mobile.android.R;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.mobile.android.util.Analytics;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.sound.TaxonSoundFactSheetController;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.sound.TaxonSoundFactSheetUI;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.observation.simple.SimpleObservationSet;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class TaxonSoundFactSheetAndroidController extends TaxonSoundFactSheetController
  {
    private static final String CLASS = TaxonSoundFactSheetAndroidController.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final TaxonSoundFactSheetUI ui;

    private SoundAdapter soundsAdapter;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TaxonSoundFactSheetAndroidController (final @Nonnull TaxonSoundFactSheetUI ui, final @Nonnull Taxon taxon)
      throws IOException
      {
        super(ui, taxon);
        this.ui = ui;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void initialize()
      {
        soundsAdapter = new SoundAdapter((Activity)ui, this, new SimpleObservationSet()); // FIXME: empty
        
        new Thread()
          {
            @Override
            public void run()
              {
                try
                  {
                    soundsAdapter = new SoundAdapter((Activity)ui, TaxonSoundFactSheetAndroidController.this, loadObservationSet());
                  }
                catch (IOException e)
                  {
                    // TODO: stop waiting
                    e.printStackTrace();
                  }
                
                ui.notifyDataLoaded();
              }
          }.start();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public SoundAdapter getSoundsAdapter()
      {
        return soundsAdapter;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected ObservationSet loadObservationSet()
      throws IOException
      {
        logger.info("loadObservationSet()");
        final String resourceName = "xenocanto-" + taxon.getId().stringValue().replaceAll(".*#", "") + ".json";
        logger.info(">>>> resourceName %s", resourceName);
        InputStream is = openResourceStream(resourceName);

        if (resourceName.endsWith(".gz"))
          {
            is = new GZIPInputStream(is);
          }

        final ObservationSet observationSet = super.loadObservationSet(is);
        is.close();
        return observationSet;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    protected Observation getObservation (final @Nonnegative int index)
      {
        return soundsAdapter.getItem(index);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected InputStream openResourceStream (final @Nonnull String resourceName)
      throws IOException
      {
        return Locator.find(AssetManager.class).open(resourceName);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void notifyStatusChanged()
      {
        ((Activity)ui).findViewById(R.id.list).post(new Runnable() // FIXME: simplify this
          {
            public void run()
              {
                soundsAdapter.notifyDataSetChanged();
              }
          });
      }

    private volatile boolean pinged;

    /*******************************************************************************************************************
     *
     * Pings the server and provides some anonymous information that will be use for statistics purpose. This should
     * be moved in the function to read news, when it will be implemented.
     *
     ******************************************************************************************************************/
    @Override
    protected void ping()
      {
        if (pinged)
          {
            return;
          }

        new Thread()
          {
            @Override
            public void run()
              {
                try
                  {
                    Analytics.createURL("http://bluebill.tidalwave.it/mobile/ping").openStream().close();
                  }
                catch (FileNotFoundException e)
                  {
                    pinged = true;
//                    e.printStackTrace();
                  }
                catch (IOException e)
                  {
//                    e.printStackTrace();
                  }
              }
          }.start();
      }
  }
